// 
// Copyright (C) 2013, Sylvain Vervoort <stocata.org>.
// stocata.org reserves the right to modify or overwrite this NinjaScript with each release.
// Version 1.3  May 2013.

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Real Time Pivot Points.
	/// </summary>
	[Description("Real Time Pivot Points.")]
	public class SVEPivotsUtcRt : Indicator
	{
		#region Variables

		private double P, S1, R1, S2, R2, S3, R3, PL, PH;
		private double LastHigh, LastLow;
		private TimeSpan utcdiff;

		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Black,	 		"PP"));
			Plots[0].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.Blue,	 		"R1"));
			Plots[1].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.Red,		 		"S1"));
			Plots[2].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.CornflowerBlue,	"R2"));
			Plots[3].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.Coral,		 	"S2"));
			Plots[4].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.SkyBlue,	 		"R3"));
			Plots[5].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.LightSalmon,		 "S3"));
			Plots[6].Pen.DashStyle 		 		= DashStyle.Dash;
			Add(new Plot(Color.LimeGreen,		"PH"));
			Add(new Plot(Color.Fuchsia,	 		"PL"));
					
			AutoScale					= false;
			Overlay						= true;
			CalculateOnBarClose			= true;
		}
		
		protected override void OnStartUp()
		{
			// Set any variables or logic you wish to do only once at start of your indicator/strategy
			utcdiff = DateTime.UtcNow - DateTime.Now;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar < 1) return; // minimum 2 bars required
			
			// If not intraday data return
			if (!Bars.BarsType.IsIntraday) return;
			// keep highest and lowest level of the previous day
			if (High[1] > LastHigh)	LastHigh = High[1];
			if (Low[1]  < LastLow) 	LastLow	 = Low[1];
			
			// show the pivots starting at UTC:0 and local time difference
			if (ToDay(Time[0].Add(utcdiff)) == ToDay(Time[1].Add(utcdiff))) 
			{
				pp.Set(P);
				r1.Set(R1);
				s1.Set(S1);
				r2.Set(R2);
				s2.Set(S2);
				r3.Set(R3);
				s3.Set(S3);
				pl.Set(PL);
				ph.Set(PH);
			}
			
			// Calculate the new day pivots on a week day
			else if(Time[0].Add(utcdiff).DayOfWeek  != DayOfWeek.Sunday)
			{ 
				P = (LastHigh + LastLow + Close[1]) / 3;
				R1 = (2*P) - LastLow;
				S1 = (2*P) - LastHigh;
				R2 = P + (LastHigh - LastLow);
				S2 = P - (LastHigh - LastLow);
				R3 = (2*P) + (LastHigh - (2*LastLow));
				S3 = (2*P) - ((2* LastHigh) - LastLow); 
				PL = LastLow;
				PH = LastHigh;
				LastLow  = Open[0]; 
				LastHigh = Open[0];
			}
		}


		#region Properties

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries pp
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries r1
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries r2
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries r3
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries s1
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries s2
		{
			get { return Values[4]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries s3
		{
			get { return Values[6]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries ph
		{
			get { return Values[7]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries pl
		{
			get { return Values[8]; }
		}
		
		#endregion
	}
}


#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SVEPivotsUtcRt[] cacheSVEPivotsUtcRt = null;

        private static SVEPivotsUtcRt checkSVEPivotsUtcRt = new SVEPivotsUtcRt();

        /// <summary>
        /// Real Time Pivot Points.
        /// </summary>
        /// <returns></returns>
        public SVEPivotsUtcRt SVEPivotsUtcRt()
        {
            return SVEPivotsUtcRt(Input);
        }

        /// <summary>
        /// Real Time Pivot Points.
        /// </summary>
        /// <returns></returns>
        public SVEPivotsUtcRt SVEPivotsUtcRt(Data.IDataSeries input)
        {
            if (cacheSVEPivotsUtcRt != null)
                for (int idx = 0; idx < cacheSVEPivotsUtcRt.Length; idx++)
                    if (cacheSVEPivotsUtcRt[idx].EqualsInput(input))
                        return cacheSVEPivotsUtcRt[idx];

            lock (checkSVEPivotsUtcRt)
            {
                if (cacheSVEPivotsUtcRt != null)
                    for (int idx = 0; idx < cacheSVEPivotsUtcRt.Length; idx++)
                        if (cacheSVEPivotsUtcRt[idx].EqualsInput(input))
                            return cacheSVEPivotsUtcRt[idx];

                SVEPivotsUtcRt indicator = new SVEPivotsUtcRt();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                SVEPivotsUtcRt[] tmp = new SVEPivotsUtcRt[cacheSVEPivotsUtcRt == null ? 1 : cacheSVEPivotsUtcRt.Length + 1];
                if (cacheSVEPivotsUtcRt != null)
                    cacheSVEPivotsUtcRt.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSVEPivotsUtcRt = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Real Time Pivot Points.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEPivotsUtcRt SVEPivotsUtcRt()
        {
            return _indicator.SVEPivotsUtcRt(Input);
        }

        /// <summary>
        /// Real Time Pivot Points.
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEPivotsUtcRt SVEPivotsUtcRt(Data.IDataSeries input)
        {
            return _indicator.SVEPivotsUtcRt(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Real Time Pivot Points.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEPivotsUtcRt SVEPivotsUtcRt()
        {
            return _indicator.SVEPivotsUtcRt(Input);
        }

        /// <summary>
        /// Real Time Pivot Points.
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEPivotsUtcRt SVEPivotsUtcRt(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SVEPivotsUtcRt(input);
        }
    }
}
#endregion
